from click.testing import CliRunner

from tests.test_base import TestBase
from src.main import cli


class TestCLI(TestBase):
    def test_topic_error(self):
        runner = CliRunner()
        result = runner.invoke(
            cli,
            [
                "apply",
                "--kafka-brokers",
                "localhost:9092",
                "--sasl-username",
                "admin",
                "--sasl-password",
                "SECRET",
                "-f",
                "tests/files/test_invalid_topic.yml",
                "--only-validate",
            ],
        )
        self.assertEqual(result.exit_code, 1)
        self.assertIn("Error: Topic name is required", result.output)

    def test_topic_default(self):
        runner = CliRunner()
        result = runner.invoke(
            cli,
            [
                "apply",
                "--kafka-brokers",
                "localhost:9092",
                "--sasl-username",
                "admin",
                "--sasl-password",
                "SECRET",
                "-f",
                "tests/files/test_topic_default.yml",
                "--only-validate",
            ],
        )
        self.assertEqual(result.exit_code, 0)
        self.assertIn(
            "topic1: Topic num_partitions is required, "
            "using default value 1",
            result.output,
        )
        self.assertIn(
            "topic1: Topic replication_factor is required, "
            "using default value 1",
            result.output,
        )
        self.assertIn(
            "topic1: Topic cleanup_policy is required, "
            "using default value 'delete'",
            result.output,
        )
        self.assertIn(
            "topic1: Topic retention_days is required, "
            "using default value 7",
            result.output,
        )

    def test_acl_resource_type_error(self):
        runner = CliRunner()
        result = runner.invoke(
            cli,
            [
                "apply",
                "--kafka-brokers",
                "localhost:9092",
                "--sasl-username",
                "admin",
                "--sasl-password",
                "SECRET",
                "-f",
                "tests/files/test_invalid_acl_resource_type.yml",
                "--only-validate",
            ],
        )
        self.assertEqual(result.exit_code, 1)
        self.assertIn("ACL resource_type is required", result.output)

    def test_acl_resource_name_error(self):
        runner = CliRunner()
        result = runner.invoke(
            cli,
            [
                "apply",
                "--kafka-brokers",
                "localhost:9092",
                "--sasl-username",
                "admin",
                "--sasl-password",
                "SECRET",
                "-f",
                "tests/files/test_invalid_acl_resource_name.yml",
                "--only-validate",
            ],
        )
        self.assertEqual(result.exit_code, 1)
        self.assertIn("ACL resource_name is required", result.output)

    def test_acl_principal_error(self):
        runner = CliRunner()
        result = runner.invoke(
            cli,
            [
                "apply",
                "--kafka-brokers",
                "localhost:9092",
                "--sasl-username",
                "admin",
                "--sasl-password",
                "SECRET",
                "-f",
                "tests/files/test_invalid_acl_principal.yml",
                "--only-validate",
            ],
        )
        self.assertEqual(result.exit_code, 1)
        self.assertIn("ACL principal is required", result.output)

    def test_acl_host_error(self):
        runner = CliRunner()
        result = runner.invoke(
            cli,
            [
                "apply",
                "--kafka-brokers",
                "localhost:9092",
                "--sasl-username",
                "admin",
                "--sasl-password",
                "SECRET",
                "-f",
                "tests/files/test_invalid_acl_host.yml",
                "--only-validate",
            ],
        )
        self.assertEqual(result.exit_code, 1)
        self.assertIn("ACL host is required", result.output)

    def test_acl_operation_error(self):
        runner = CliRunner()
        result = runner.invoke(
            cli,
            [
                "apply",
                "--kafka-brokers",
                "localhost:9092",
                "--sasl-username",
                "admin",
                "--sasl-password",
                "SECRET",
                "-f",
                "tests/files/test_invalid_acl_operation.yml",
                "--only-validate",
            ],
        )
        self.assertEqual(result.exit_code, 1)
        self.assertIn("ACL operation is required", result.output)

    def test_acl_permission_type_error(self):
        runner = CliRunner()
        result = runner.invoke(
            cli,
            [
                "apply",
                "--kafka-brokers",
                "localhost:9092",
                "--sasl-username",
                "admin",
                "--sasl-password",
                "SECRET",
                "-f",
                "tests/files/test_invalid_acl_permission_type.yml",
                "--only-validate",
            ],
        )
        self.assertEqual(result.exit_code, 1)
        self.assertIn("ACL permission_type is required", result.output)

    def test_acl_pattern_type_error(self):
        runner = CliRunner()
        result = runner.invoke(
            cli,
            [
                "apply",
                "--kafka-brokers",
                "localhost:9092",
                "--sasl-username",
                "admin",
                "--sasl-password",
                "SECRET",
                "-f",
                "tests/files/test_invalid_acl_pattern_type.yml",
                "--only-validate",
            ],
        )
        self.assertEqual(result.exit_code, 1)
        self.assertIn("Invalid pattern_type: INVALID", result.output)



from click import ClickException, option, group, Choice
from dotenv import load_dotenv

from .command import Command

load_dotenv()


@group()
def cli():
    pass


@cli.command(name="apply")
@option(
    "--kafka-brokers",
    envvar="KAFKA_BROKERS",
    default=None,
    help="Kafka server to connect to use.",
)
@option(
    "--security-protocol",
    envvar="KAFKA_SECURITY_PROTOCOL",
    default="PLAINTEXT",
    type=Choice(["PLAINTEXT", "SASL_PLAINTEXT"]),
    help="Kafka Security protocol to use.",
)
@option(
    "--sasl-mechanism",
    envvar="KAFKA_SASL_MECHANISM",
    default="PLAIN",
    type=Choice(["PLAIN", "SCRAM-SHA-256", "SCRAM-SHA-512"]),
    help="SASL mechanism to use.",
)
@option(
    "--sasl-username",
    envvar="KAFKA_SASL_USERNAME",
    default=None,
    help="SASL username to use.",
)
@option(
    "--sasl-password",
    envvar="KAFKA_SASL_PASSWORD",
    default="",
    help="SASL password to use.",
)
@option("--file", "-f", help="File path to validate")
@option(
    "--only-validate",
    default=False,
    is_flag=True,
    help="Only validate the file",
)
def apply(
    kafka_brokers: str,
    security_protocol: str,
    sasl_mechanism: str,
    sasl_username: str,
    sasl_password: str,
    file: str,
    only_validate: bool,
):
    """Apply configuration file with topics and ACL to Kafka.
    Args:
        kafka_brokers (str): Kafka server to connect to use.
        security_protocol (str): Kafka Security protocol to use.
        sasl_mechanism (str): SASL mechanism to use.
        sasl_username (str): SASL username to use.
        sasl_password (str): SASL password to use.
        file (str): File path to validate.
        only_validate (bool): Only validate the file.
    """
    try:
        cmd: Command = Command(
            kafka_brokers=kafka_brokers,
            security_protocol=security_protocol,
            sasl_mechanism=sasl_mechanism,
            sasl_username=sasl_username,
            sasl_password=sasl_password,
            file=file,
            only_validate=only_validate,
        )
        cmd.run()
    except Exception as e:
        raise ClickException(str(e))


if __name__ == "__main__":
    cli()



import os
import json

from click import echo
import yaml


class ReadFile:
    def __init__(self, filename):
        self.filename = filename

    def read(self) -> dict:
        """Read a file and return the content
        Returns:
            dict: Content of the file
        """
        try:
            if not os.path.exists(self.filename):
                raise ValueError(f"File {self.filename} does not exists")

            if self.filename.endswith(".json"):
                echo("JSON file detected")
                with open(self.filename, "r") as f:
                    data = json.load(f)
            elif self.filename.endswith(".yaml") or self.filename.endswith(
                ".yml"
            ):
                echo("YAML file detected")
                with open(self.filename, "r") as f:
                    data = yaml.safe_load(f)
            else:
                raise ValueError(
                    "File extension not supported (only .json and .yaml/.yml"
                )

            return data
        except Exception as e:
            raise ValueError(f"Error reading file {self.filename}: {e}")



from typing import List

from click import echo
from kafka.admin import (
    KafkaAdminClient,
    NewTopic,
    ConfigResource,
    ConfigResourceType,
    ACL,
    ACLFilter,
    ACLOperation,
    ACLPermissionType,
    ResourcePatternFilter,
    ResourceType,
    ACLResourcePatternType,
)


class KafkaHandler:
    """This class is responsible for handling Kafka topics and ACLs."""

    def __init__(
        self,
        kafka_brokers: str,
        security_protocol: str,
        sasl_mechanism: str,
        sasl_username: str,
        sasl_password: str,
    ):
        """Create a KafkaTopic instance.
        Args:
            kafka_brokers (str): Kafka brokers
            security_protocol (str): Security protocol
            sasl_mechanism (str): SASL mechanism
            sasl_username (str): SASL username
            sasl_password (str): SASL password
        """
        self.admin_client: KafkaAdminClient = KafkaAdminClient(
            bootstrap_servers=kafka_brokers,
            security_protocol=security_protocol,
            sasl_mechanism=sasl_mechanism,
            sasl_plain_username=sasl_username,
            sasl_plain_password=sasl_password,
        )

    def create_topics(self, topics: List[NewTopic]):
        """Create topics.
        Args:
            topics (List[NewTopic]): List of topics to create
        """
        topic_metadata = self.admin_client.list_topics()
        for topic in topics:
            if topic.name in topic_metadata:
                fs = self.admin_client.alter_configs(
                    [
                        ConfigResource(
                            resource_type=ConfigResourceType.TOPIC,
                            name=topic.name,
                            configs=topic.topic_configs,
                        )
                    ]
                )
                if fs:
                    echo(f"Topic {topic.name} updated")
            else:
                fs = self.admin_client.create_topics([topic])
                if fs:
                    echo(f"Topic {topic.name} created")

    def create_acls(self, acls: List[ACL]):
        """Create new ACLs and remove old ones.
        Args:
            acls (List[ACL]): List of ACLs to create
        """
        filter_acls: ACLFilter = ACLFilter(
            principal=None,
            host=None,
            operation=ACLOperation.ANY,
            permission_type=ACLPermissionType.ANY,
            resource_pattern=ResourcePatternFilter(
                resource_type=ResourceType.ANY,
                resource_name=None,
                pattern_type=ACLResourcePatternType.ANY,
            ),
        )
        kafka_acls = self.admin_client.describe_acls(acl_filter=filter_acls)

        for acl in acls:
            filter_acls = ACLFilter(
                principal=acl.principal,
                host=acl.host,
                operation=acl.operation,
                permission_type=acl.permission_type,
                resource_pattern=ResourcePatternFilter(
                    resource_type=acl.resource_pattern.resource_type,
                    resource_name=acl.resource_pattern.resource_name,
                    pattern_type=acl.resource_pattern.pattern_type,
                ),
            )
            file_acls = self.admin_client.describe_acls(acl_filter=filter_acls)
            if len(file_acls[0]) > 0:
                echo(f"ACL {acl.principal} already exists")
            else:
                self.admin_client.create_acls([acl])
                echo(f"ACL {acl.principal} created")

            for kafka_acl in kafka_acls[0]:
                if kafka_acl == acl:
                    kafka_acls[0].remove(kafka_acl)

        for kafka_acl in kafka_acls[0]:
            self.admin_client.delete_acls([kafka_acl])
            echo(f"ACL {kafka_acl.principal} removed")

    def close(self):
        """Close the KafkaAdminClient."""
        self.admin_client.close()



import os
import unittest


class TestBase(unittest.TestCase):
    def __init_subclass__(cls):
        if "KAFKA_BROKERS" in os.environ:
            del os.environ["KAFKA_BROKERS"]
        if "KAFKA_SECURITY_PROTOCOL" in os.environ:
            del os.environ["KAFKA_SECURITY_PROTOCOL"]
        if "KAFKA_SASL_MECHANISM" in os.environ:
            del os.environ["KAFKA_SASL_MECHANISM"]
        if "KAFKA_SASL_USERNAME" in os.environ:
            del os.environ["KAFKA_SASL_USERNAME"]
        if "KAFKA_SASL_PASSWORD" in os.environ:
            del os.environ["KAFKA_SASL_PASSWORD"]



from unittest import mock

from click.testing import CliRunner
from kafka.admin import KafkaAdminClient

from tests.test_base import TestBase
from src.main import cli


class TestCLI(TestBase):
    def test_cli(self):
        runner = CliRunner()
        result = runner.invoke(cli)
        self.assertEqual(result.exit_code, 0)

    def test_command_apply(self):
        runner = CliRunner()
        result = runner.invoke(cli, ["apply"])
        self.assertEqual(result.exit_code, 0)

    def test_command_apply_file_error(self):
        runner = CliRunner()
        result = runner.invoke(
            cli,
            [
                "apply",
                "--kafka-brokers",
                "localhost:9092",
                "--sasl-username",
                "admin",
                "--sasl-password",
                "SECRET",
                "-f",
                "tests/test.yml",
                "--only-validate",
            ],
        )
        self.assertEqual(result.exit_code, 1)
        self.assertIn(
            "File tests/test.yml does not exists", result.output
        )

    def test_command_apply_file_json(self):
        runner = CliRunner()
        result = runner.invoke(
            cli,
            [
                "apply",
                "--kafka-brokers",
                "localhost:9092",
                "--sasl-username",
                "admin",
                "--sasl-password",
                "SECRET",
                "-f",
                "tests/files/test.json",
                "--only-validate",
            ],
        )
        self.assertEqual(result.exit_code, 0)
        self.assertIn("JSON file detected", result.output)

    def test_command_apply_file_yaml(self):
        runner = CliRunner()
        result = runner.invoke(
            cli,
            [
                "apply",
                "--kafka-brokers",
                "localhost:9092",
                "--sasl-username",
                "admin",
                "--sasl-password",
                "SECRET",
                "-f",
                "tests/files/test.yml",
                "--only-validate",
            ],
        )
        self.assertEqual(result.exit_code, 0)
        self.assertIn("YAML file detected", result.output)

    def test_command_apply_file_not_supported(self):
        runner = CliRunner()
        result = runner.invoke(
            cli,
            [
                "apply",
                "--kafka-brokers",
                "localhost:9092",
                "--sasl-username",
                "admin",
                "--sasl-password",
                "SECRET",
                "-f",
                "tests/__init__.py",
                "--only-validate",
            ],
        )
        self.assertEqual(result.exit_code, 1)
        self.assertIn(
            "File extension not supported (only .json and .yaml/.yml",
            result.output,
        )

    def test_command_apply_file_validate(self):
        runner = CliRunner()
        result = runner.invoke(
            cli,
            [
                "apply",
                "--kafka-brokers",
                "localhost:9092",
                "--sasl-username",
                "admin",
                "--sasl-password",
                "SECRET",
                "-f",
                "tests/files/test.yml",
                "--only-validate",
            ],
        )
        self.assertEqual(result.exit_code, 0)
        self.assertIn("File tests/files/test.yml is valid.", result.output)

    def test_command_apply_invalid_cleanup_policy(self):
        runner = CliRunner()
        result = runner.invoke(
            cli,
            [
                "apply",
                "--kafka-brokers",
                "localhost:9092",
                "--sasl-username",
                "admin",
                "--sasl-password",
                "SECRET",
                "-f",
                "tests/files/test_invalid_01.yml",
                "--only-validate",
            ],
        )
        self.assertEqual(result.exit_code, 1)
        self.assertIn(
            "Topic cleanup_policy must be 'delete' or 'compact'", result.output
        )

    def test_command_apply_invalid_resource_type(self):
        runner = CliRunner()
        result = runner.invoke(
            cli,
            [
                "apply",
                "--kafka-brokers",
                "localhost:9092",
                "--sasl-username",
                "admin",
                "--sasl-password",
                "SECRET",
                "-f",
                "tests/files/test_invalid_02.yml",
                "--only-validate",
            ],
        )
        self.assertEqual(result.exit_code, 1)
        self.assertIn(
            "Invalid resource_type: INVALID, valid values are: UNKNOWN",
            result.output,
        )

    def test_command_apply_invalid_operation(self):
        runner = CliRunner()
        result = runner.invoke(
            cli,
            [
                "apply",
                "--kafka-brokers",
                "localhost:9092",
                "--sasl-username",
                "admin",
                "--sasl-password",
                "SECRET",
                "-f",
                "tests/files/test_invalid_03.yml",
                "--only-validate",
            ],
        )
        self.assertEqual(result.exit_code, 1)
        self.assertIn(
            "Invalid operation: INVALID, valid values are: ANY, ALL, READ",
            result.output,
        )

    def test_command_apply_invalid_permission_type(self):
        runner = CliRunner()
        result = runner.invoke(
            cli,
            [
                "apply",
                "--kafka-brokers",
                "localhost:9092",
                "--sasl-username",
                "admin",
                "--sasl-password",
                "SECRET",
                "-f",
                "tests/files/test_invalid_04.yml",
                "--only-validate",
            ],
        )
        self.assertEqual(result.exit_code, 1)
        self.assertIn(
            "Invalid permission_type: INVALID, valid values are: ANY, DENY",
            result.output,
        )

    def test_command_apply_invalid_security_protocol(self):
        runner = CliRunner()
        result = runner.invoke(
            cli,
            [
                "apply",
                "--security-protocol",
                "INVALID",
                "-f",
                "tests/files/test.yml",
                "--only-validate",
            ],
        )
        self.assertEqual(result.exit_code, 2)
        self.assertIn(
            "Invalid value for '--security-protocol': 'INVALID' "
            "is not one of 'PLAINTEXT', 'SASL_PLAINTEXT'",
            result.output,
        )

    def test_command_apply_invalid_sasl_mechanism(self):
        runner = CliRunner()
        result = runner.invoke(
            cli,
            [
                "apply",
                "--sasl-mechanism",
                "INVALID",
                "-f",
                "tests/files/test.yml",
                "--only-validate",
            ],
        )
        self.assertEqual(result.exit_code, 2)
        self.assertIn(
            "Invalid value for '--sasl-mechanism': 'INVALID' is not one of "
            "'PLAIN', 'SCRAM-SHA-256', 'SCRAM-SHA-512'.",
            result.output,
        )

    @mock.patch.object(KafkaAdminClient, "__init__", return_value=None)
    @mock.patch.object(KafkaAdminClient, "list_topics")
    @mock.patch.object(KafkaAdminClient, "create_topics")
    @mock.patch.object(KafkaAdminClient, "create_acls")
    @mock.patch.object(KafkaAdminClient, "describe_acls")
    @mock.patch.object(KafkaAdminClient, "delete_acls")
    @mock.patch.object(KafkaAdminClient, "close")
    def test_command_apply_ok(
        self,
        mock_close,
        mock_delete_acls,
        mock_describe_acls,
        mock_create_acls,
        mock_create_topics,
        mock_list_topics,
        mock_kafka_admin,
    ):
        mock_close.return_value = None
        mock_delete_acls.return_value = True
        mock_describe_acls.return_value = ([], 1)
        mock_create_acls.return_value = True
        mock_create_topics.return_value = True
        mock_list_topics.return_value = []
        mock_kafka_admin.return_value = None
        runner = CliRunner()
        result = runner.invoke(
            cli,
            [
                "apply",
                "--kafka-brokers",
                "localhost:9092",
                "--security-protocol",
                "PLAINTEXT",
                "--sasl-mechanism",
                "PLAIN",
                "--sasl-username",
                "admin",
                "--sasl-password",
                "SECRET",
                "-f",
                "tests/files/test.yml",
            ],
        )
        self.assertEqual(result.exit_code, 0)
        self.assertIn("Applying file...", result.output)
        self.assertIn("2 topics found.", result.output)
        self.assertIn("2 ACLs found.", result.output)
        self.assertIn("File synced successfully.", result.output)

    @mock.patch.object(KafkaAdminClient, "__init__", return_value=None)
    @mock.patch.object(KafkaAdminClient, "list_topics")
    @mock.patch.object(KafkaAdminClient, "alter_configs")
    @mock.patch.object(KafkaAdminClient, "create_acls")
    @mock.patch.object(KafkaAdminClient, "describe_acls")
    @mock.patch.object(KafkaAdminClient, "delete_acls")
    @mock.patch.object(KafkaAdminClient, "close")
    def test_command_apply_update_ok(
        self,
        mock_close,
        mock_delete_acls,
        mock_describe_acls,
        mock_create_acls,
        mock_alter_configs,
        mock_list_topics,
        mock_kafka_admin,
    ):
        mock_close.return_value = None
        mock_delete_acls.return_value = True
        mock_describe_acls.return_value = ([], 1)
        mock_create_acls.return_value = True
        mock_alter_configs.return_value = True
        mock_list_topics.return_value = [
            "topic1",
            "topic2",
        ]
        mock_kafka_admin.return_value = None
        runner = CliRunner()
        result = runner.invoke(
            cli,
            [
                "apply",
                "--kafka-brokers",
                "localhost:9092",
                "--security-protocol",
                "PLAINTEXT",
                "--sasl-mechanism",
                "PLAIN",
                "--sasl-username",
                "admin",
                "--sasl-password",
                "SECRET",
                "-f",
                "tests/files/test.yml",
            ],
        )
        self.assertEqual(result.exit_code, 0)
        self.assertIn("Applying file...", result.output)
        self.assertIn("2 topics found.", result.output)
        self.assertIn("2 ACLs found.", result.output)
        self.assertIn("File synced successfully.", result.output)

    def test_command_validate_brokers(self):
        runner = CliRunner()
        result = runner.invoke(
            cli,
            [
                "apply",
                "-f",
                "tests/files/test.yml",
            ],
        )
        self.assertEqual(result.exit_code, 0)
        self.assertIn("Kafka brokers not found.", result.output)

    def test_command_validate_username(self):
        runner = CliRunner()
        result = runner.invoke(
            cli,
            [
                "apply",
                "--kafka-brokers",
                "localhost:9092",
                "-f",
                "tests/files/test.yml",
            ],
        )
        self.assertEqual(result.exit_code, 0)
        self.assertIn("SASL username not found.", result.output)

    def test_command_validate_password(self):
        runner = CliRunner()
        result = runner.invoke(
            cli,
            [
                "apply",
                "--kafka-brokers",
                "localhost:9092",
                "--sasl-username",
                "admin",
                "-f",
                "tests/files/test.yml",
            ],
        )
        self.assertEqual(result.exit_code, 0)
        self.assertIn("SASL password not found.", result.output)



from typing import Dict, List

from click import echo
from kafka.admin import (
    NewTopic,
    ACL,
    ResourcePattern,
    ResourceType,
    ACLPermissionType,
    ACLOperation,
    ACLResourcePatternType,
)


class Validator:
    """ This class validates the configuration file. """
    @staticmethod
    def check_topics(data: Dict) -> List[NewTopic]:
        """Check topics, return a list of NewTopic objects.
        Args:
            data (Dict): Data from file
        Returns:
            List[NewTopic]: List of NewTopic objects
        """
        topics: List[NewTopic] = []
        topic_name: str
        num_partitions: int
        replication_factor: int
        cleanup_policy: str
        retention_ms: int
        if not data.get("topics", []):
            echo("No topics found")

        for topic in data.get("topics", []):

            if not topic.get("name", None):
                raise Exception("Topic name is required")
            else:
                topic_name = topic["name"]

            if not topic.get("num_partitions", None):
                echo(
                    f"{topic_name}: Topic num_partitions is required, "
                    f"using default value 1"
                )
                num_partitions = 1
            else:
                num_partitions = topic["num_partitions"]

            if not topic.get("replication_factor", None):
                replication_factor = 1
                echo(
                    f"{topic_name}: Topic replication_factor is required, "
                    f"using default value 1"
                )
            else:
                replication_factor = topic["replication_factor"]

            if not topic.get("cleanup_policy", None):
                cleanup_policy = "delete"
                echo(
                    f"{topic_name}: Topic cleanup_policy is required, "
                    f"using default value 'delete'"
                )
            else:
                if topic["cleanup_policy"] not in ["delete", "compact"]:
                    raise Exception(
                        f"{topic_name}: Topic cleanup_policy must be "
                        f"'delete' or 'compact'"
                    )
                else:
                    cleanup_policy = topic["cleanup_policy"]

            if not topic.get("retention_days", None):
                retention_ms = 7 * 24 * 60 * 60 * 1000
                echo(
                    f"{topic_name}: Topic retention_days is required, "
                    f"using default value 7"
                )
            else:
                retention_ms = topic["retention_days"] * 24 * 60 * 60 * 1000

            topics.append(
                NewTopic(
                    name=topic_name,
                    num_partitions=num_partitions,
                    replication_factor=replication_factor,
                    topic_configs={
                        "cleanup.policy": cleanup_policy,
                        "retention.ms": retention_ms,
                    },
                )
            )

        return topics

    @staticmethod
    def check_acls(data: Dict) -> List[ACL]:
        """Check acls, return a list of ACL objects.
        Args:
            data (Dict): Data from file
        Returns:
            List[ACL]: List of ACL objects
        """
        acls: List[ACL] = []
        resource_type: ResourceType
        resource_name: str
        principal: str
        host: str
        operation: ACLOperation
        permission_type: ACLPermissionType
        pattern_type: ACLResourcePatternType
        if not data.get("acls", []):
            echo("No acls found")

        for acl in data.get("acls", []):

            if not acl.get("resource_type", None):
                raise Exception("ACL resource_type is required")
            else:
                resource_type = Validator.get_enum_value(
                    enum_class=ResourceType, value=acl["resource_type"]
                )
                if not resource_type:
                    raise Exception(
                        f"Invalid resource_type: {acl['resource_type']}, "
                        f"valid values are: UNKNOWN, ANY, CLUSTER, TOPIC,"
                        f"DELEGATION_TOKEN, GROUP, TRANSACTIONAL_ID."
                    )

            if not acl.get("resource_name", None):
                raise Exception("ACL resource_name is required")
            else:
                resource_name = acl["resource_name"]

            if not acl.get("principal", None):
                raise Exception("ACL principal is required")
            else:
                principal = acl["principal"]

            if not acl.get("host", None):
                raise Exception("ACL host is required")
            else:
                host = acl["host"]

            if not acl.get("operation", None):
                raise Exception("ACL operation is required")
            else:
                operation = Validator.get_enum_value(
                    enum_class=ACLOperation, value=acl["operation"]
                )
                if not operation:
                    raise Exception(
                        f"Invalid operation: {acl['operation']}, "
                        f"valid values are: ANY, ALL, READ, WRITE, CREATE, "
                        f"DELETE, ALTER, DESCRIBE, CLUSTER_ACTION, "
                        f"DESCRIBE_CONFIGS, ALTER_CONFIGS, IDEMPOTENT_WRITE"
                    )

            if not acl.get("permission_type", None):
                raise Exception("ACL permission_type is required")
            else:
                permission_type = Validator.get_enum_value(
                    enum_class=ACLPermissionType, value=acl["permission_type"]
                )
                if not permission_type:
                    raise Exception(
                        f"Invalid permission_type: {acl['permission_type']}, "
                        f"valid values are: ANY, DENY, ALLOW"
                    )

            if not acl.get("pattern_type", None):
                pattern_type = ACLResourcePatternType.LITERAL
            else:
                pattern_type = Validator.get_enum_value(
                    enum_class=ACLResourcePatternType, value=acl["pattern_type"]
                )
                if not pattern_type:
                    raise Exception(
                        f"Invalid pattern_type: {acl['pattern_type']}, "
                        f"valid values are: ANY, MATCH, LITERAL, PREFIXED"
                    )

            acls.append(
                ACL(
                    principal=principal,
                    host=host,
                    operation=operation,
                    permission_type=permission_type,
                    resource_pattern=ResourcePattern(
                        resource_type=resource_type,
                        resource_name=resource_name,
                        pattern_type=pattern_type,
                    ),
                )
            )

        return acls

    @staticmethod
    def get_enum_value(enum_class: any, value: str) -> any:
        """Check if value is valid enum.
        Args:
            enum_class: Enum class
            value (str): Value to check
        Returns:
            any: Enum member or False
        """
        try:
            member = enum_class[value]
            return member
        except KeyError:
            return False



from typing import List

from click import echo
from kafka.admin import NewTopic, ACL

from .read_file import ReadFile
from .validator import Validator
from .kafka_handler import KafkaHandler


class Command:
    """This class is responsible for running the command."""

    def __init__(
        self,
        kafka_brokers: str,
        security_protocol: str,
        sasl_mechanism: str,
        sasl_username: str,
        sasl_password: str,
        file: str,
        only_validate: bool,
    ):
        """
        Create a Command instance.
        Args:
            kafka_brokers: Kafka brokers
            security_protocol: Security protocol
            sasl_mechanism: SASL mechanism
            sasl_username: SASL username
            sasl_password: SASL password
            file: File to read
            only_validate: Only validate file
        """
        self.kafka_brokers: str = kafka_brokers
        self.security_protocol: str = security_protocol
        self.sasl_mechanism: str = sasl_mechanism
        self.sasl_username: str = sasl_username
        self.sasl_password: str = sasl_password
        self.file: str = file
        self.only_validate: bool = only_validate

    def run(self):
        """Run the command."""
        if self.only_validate:
            echo(f"Validating file {self.file}...")
        else:
            echo("Applying file...")

            echo(f"Kafka brokers: {self.kafka_brokers}")
            echo(f"Security protocol: {self.security_protocol}")
            echo(f"SASL mechanism: {self.sasl_mechanism}")
            echo(f"SASL username: {self.sasl_username}")
            echo(
                f"SASL password: {self.sasl_password.replace(self.sasl_password, '*' * len(self.sasl_password))}"
            )

        if self.file is None:
            echo("File not found.")
            return

        echo(f"File: {self.file}")
        read_file = ReadFile(self.file)
        data = read_file.read()
        # Validate file
        validator = Validator()
        topics: List[NewTopic] = validator.check_topics(data=data)
        acls: List[ACL] = validator.check_acls(data=data)

        echo(f"{len(topics)} topics found.")
        echo(f"{len(acls)} ACLs found.")

        if self.only_validate:
            echo(f"File {self.file} is valid.")
            return

        if self.kafka_brokers is None:
            echo("Kafka brokers not found.")
            return

        if self.sasl_username is None:
            echo("SASL username not found.")
            return

        if self.sasl_password == "" or self.sasl_password is None:  # nosec
            echo("SASL password not found.")
            return

        kafka: KafkaHandler = KafkaHandler(
            kafka_brokers=self.kafka_brokers,
            security_protocol=self.security_protocol,
            sasl_mechanism=self.sasl_mechanism,
            sasl_username=self.sasl_username,
            sasl_password=self.sasl_password,
        )

        # Create topics
        if len(topics) > 0:
            kafka.create_topics(topics=topics)

        # Create ACLs
        if len(acls) > 0:
            kafka.create_acls(acls=acls)

        kafka.close()
        echo("File synced successfully.")
